@@ -203,6 +203,7 @@ ephy_embed_shell_update_overview_urls (EphyEmbedShell *shell)
   query->sort_type = EPHY_HISTORY_SORT_MOST_VISITED;
   query->limit = EPHY_ABOUT_OVERVIEW_MAX_ITEMS;
   query->ignore_hidden = TRUE;
+  query->ignore_local = TRUE;
 
   ephy_history_service_query_urls (shell->priv->global_history_service, query, NULL,
                                    (EphyHistoryJobCallback) history_service_query_urls_cb,
@@ -394,10 +395,6 @@ ephy_embed_shell_set_thumbanil_path (EphyEmbedShell *shell,
 {
   GList *l;
 
-  ephy_history_service_set_url_thumbnail_time (shell->priv->global_history_service,
-                                               url, mtime,
-                                               NULL, NULL, NULL);
-
   for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
     EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
 
@@ -418,10 +415,16 @@ ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell)
 
   if (shell->priv->global_history_service == NULL) {
     char *filename;
+    EphySQLiteConnectionMode mode;
+
+    if (shell->priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO ||
+        shell->priv->mode == EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER)
+      mode = EPHY_SQLITE_CONNECTION_MODE_READ_ONLY;
+    else
+      mode = EPHY_SQLITE_CONNECTION_MODE_READWRITE;
 
     filename = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
-    shell->priv->global_history_service = ephy_history_service_new (filename,
-                                                                    shell->priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO);
+    shell->priv->global_history_service = ephy_history_service_new (filename, mode);
     g_free (filename);
     g_return_val_if_fail (shell->priv->global_history_service, NULL);
     g_signal_connect (shell->priv->global_history_service, "urls-visited",
@@ -444,6 +447,17 @@ ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell)
   return G_OBJECT (shell->priv->global_history_service);
 }
 
+static void
+snapshot_saved_cb (EphySnapshotService *service,
+                   const char          *url,
+                   gint64               mtime,
+                   EphyEmbedShell      *shell)
+{
+  ephy_history_service_set_url_thumbnail_time (EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (shell)),
+                                               url, mtime,
+                                               NULL, NULL, NULL);
+}
+
 /**
  * ephy_embed_shell_get_encodings:
  * @shell: the #EphyEmbedShell
@@ -759,6 +773,10 @@ ephy_embed_shell_constructed (GObject *object)
     ephy_embed_shell_create_web_context (embed_shell);
     embed_shell->priv->user_content = webkit_user_content_manager_new ();
   }
+
+  g_signal_connect_object (ephy_snapshot_service_get_default (),
+                           "snapshot-saved", G_CALLBACK (snapshot_saved_cb),
+                           embed_shell, 0);
 }
 
 static void
