@@ -228,15 +228,6 @@ ephy_embed_statusbar_pop (EphyEmbed *embed, guint context_id)
   ephy_embed_statusbar_update (embed, msg ? msg->text : NULL);
 }
 
-static void
-ephy_embed_destroy_top_widgets (EphyEmbed *embed)
-{
-  GSList *iter;
-
-  for (iter = embed->priv->destroy_on_transition_list; iter; iter = iter->next)
-    gtk_widget_destroy (GTK_WIDGET (iter->data));
-}
-
 static void
 remove_from_destroy_list_cb (GtkWidget *widget, EphyEmbed *embed)
 {
@@ -248,7 +239,20 @@ remove_from_destroy_list_cb (GtkWidget *widget, EphyEmbed *embed)
 }
 
 static void
-ephy_embed_set_title (EphyEmbed *embed,
+ephy_embed_destroy_top_widgets (EphyEmbed *embed)
+{
+  GSList *iter;
+
+  for (iter = embed->priv->destroy_on_transition_list; iter; iter = iter->next) {
+    g_signal_handlers_disconnect_by_func (iter->data, remove_from_destroy_list_cb, embed);
+    gtk_widget_destroy (GTK_WIDGET (iter->data));
+  }
+
+  embed->priv->destroy_on_transition_list = NULL;
+}
+
+static void
+ephy_embed_set_title (EphyEmbed  *embed,
                       const char *title)
 {
   EphyEmbedPrivate *priv = embed->priv;
@@ -294,11 +298,12 @@ load_changed_cb (WebKitWebView *web_view,
   case WEBKIT_LOAD_COMMITTED:
     ephy_embed_destroy_top_widgets (embed);
     break;
-  case WEBKIT_LOAD_FINISHED:
-    if (ephy_web_view_get_is_blank (EPHY_WEB_VIEW (web_view)) ||
-        !webkit_web_view_get_title (web_view))
+  case WEBKIT_LOAD_FINISHED: {
+    const char *title = webkit_web_view_get_title (web_view);
+    if (ephy_web_view_get_is_blank (EPHY_WEB_VIEW (web_view)) || !title || !*title)
       ephy_embed_set_title (embed, NULL);
     break;
+  }
   default:
     break;
   }
