@@ -44,10 +44,12 @@ struct _UriTesterPrivate
   char *data_dir;
 
   GHashTable *pattern;
+  GHashTable *keys;
   GHashTable *optslist;
   GHashTable *urlcache;
 
   GHashTable *whitelisted_pattern;
+  GHashTable *whitelisted_keys;
   GHashTable *whitelisted_optslist;
   GHashTable *whitelisted_urlcache;
 
@@ -111,35 +113,12 @@ uri_tester_get_fileuri_for_url (UriTester *tester,
   return uri;
 }
 
-typedef struct {
-  UriTester *tester;
-  char *dest_uri;
-} RetrieveFilterAsyncData;
-
-static void
-uri_tester_retrieve_filter_finished (GFile *src,
-                                     GAsyncResult *result,
-                                     RetrieveFilterAsyncData *data)
-{
-  GError *error = NULL;
-
-  if (!g_file_copy_finish (src, result, &error)) {
-    LOG ("Error retrieving filter: %s\n", error->message);
-    g_error_free (error);
-  } else
-    uri_tester_parse_file_at_uri (data->tester, data->dest_uri);
-
-  g_object_unref (data->tester);
-  g_free (data->dest_uri);
-  g_slice_free (RetrieveFilterAsyncData, data);
-}
-
 static void
 uri_tester_retrieve_filter (UriTester *tester, const char *url, const char *fileuri)
 {
   GFile *src;
   GFile *dest;
-  RetrieveFilterAsyncData *data;
+  GError *error = NULL;
 
   g_return_if_fail (IS_URI_TESTER (tester));
   g_return_if_fail (url != NULL);
@@ -148,16 +127,17 @@ uri_tester_retrieve_filter (UriTester *tester, const char *url, const char *file
   src = g_file_new_for_uri (url);
   dest = g_file_new_for_uri (fileuri);
 
-  data = g_slice_new (RetrieveFilterAsyncData);
-  data->tester = g_object_ref (tester);
-  data->dest_uri = g_file_get_uri (dest);
+  g_file_copy (src, dest,
+               G_FILE_COPY_OVERWRITE,
+               NULL, NULL, NULL,
+               &error);
 
-  g_file_copy_async (src, dest,
-                     G_FILE_COPY_OVERWRITE,
-                     G_PRIORITY_DEFAULT,
-                     NULL, NULL, NULL,
-                     (GAsyncReadyCallback)uri_tester_retrieve_filter_finished,
-                     data);
+  if (error != NULL) {
+    g_warning ("Error retrieving adblock filter: %s\n", error->message);
+    g_error_free (error);
+  } else {
+    uri_tester_parse_file_at_uri (tester, fileuri);
+  }
 
   g_object_unref (src);
   g_object_unref (dest);
@@ -257,7 +237,7 @@ uri_tester_load_filters (UriTester *tester)
 
       if (error)
         {
-          LOG ("Error loading filters from %s: %s", filepath, error->message);
+          g_warning ("Error loading filters from %s: %s", filepath, error->message);
           g_error_free (error);
         }
 
@@ -350,6 +330,54 @@ uri_tester_is_matched_by_pattern (UriTester  *tester,
   return FALSE;
 }
 
+static inline gboolean
+uri_tester_is_matched_by_key (UriTester  *tester,
+                              const char *opts,
+                              const char *req_uri,
+                              const char *page_uri,
+                              gboolean whitelist)
+{
+  UriTesterPrivate *priv = NULL;
+  char *uri;
+  int len;
+  int pos = 0;
+  GList *regex_bl = NULL;
+  GString *guri;
+  gboolean ret = FALSE;
+  char sig[SIGNATURE_SIZE + 1];
+  GHashTable *keys;
+
+  priv = tester->priv;
+
+  keys = priv->keys;
+  if (whitelist)
+    keys = priv->whitelisted_keys;
+
+  memset (&sig[0], 0, sizeof (sig));
+  /* Signatures are made on pattern, so we need to convert url to a pattern as well */
+  guri = uri_tester_fixup_regexp ("", (char*)req_uri);
+  uri = guri->str;
+  len = guri->len;
+
+  for (pos = len - SIGNATURE_SIZE; pos >= 0; pos--)
+    {
+      GRegex *regex;
+      strncpy (sig, uri + pos, SIGNATURE_SIZE);
+      regex = g_hash_table_lookup (keys, sig);
+
+      /* Dont check if regex is already blacklisted */
+      if (!regex || g_list_find (regex_bl, regex))
+        continue;
+      ret = uri_tester_check_rule (tester, regex, sig, req_uri, page_uri, whitelist);
+      if (ret)
+        break;
+      regex_bl = g_list_prepend (regex_bl, regex);
+    }
+  g_string_free (guri, TRUE);
+  g_list_free (regex_bl);
+  return ret;
+}
+
 static gboolean
 uri_tester_is_matched (UriTester  *tester,
                        const char *opts,
@@ -371,6 +399,13 @@ uri_tester_is_matched (UriTester  *tester,
   if ((value = g_hash_table_lookup (urlcache, req_uri)))
     return (value[0] != '0') ? TRUE : FALSE;
 
+  /* Look for a match either by key or by pattern. */
+  if (uri_tester_is_matched_by_key (tester, opts, req_uri, page_uri, whitelist))
+    {
+      g_hash_table_insert (urlcache, g_strdup (req_uri), g_strdup("1"));
+      return TRUE;
+    }
+
   /* Matching by pattern is pretty expensive, so do it if needed only. */
   if (uri_tester_is_matched_by_pattern (tester, req_uri, page_uri, whitelist))
     {
@@ -386,7 +421,6 @@ static GString *
 uri_tester_fixup_regexp (const char *prefix, char *src)
 {
   GString *str;
-  int len = 0;
 
   if (!src)
     return NULL;
@@ -406,21 +440,27 @@ uri_tester_fixup_regexp (const char *prefix, char *src)
         case '*':
           g_string_append (str, ".*");
           break;
-          /*case '.':
-            g_string_append (str, "\\.");
-            break;*/
+        case '^':
+          g_string_append (str, "([^a-zA-Z\\d]|[_\\-\\.%])");
+          break;
+        case '|':
+          if (src[1] == '\0')
+            g_string_append (str, "$");
+          else
+            g_string_append (str, "\\|");
+          break;
+        case '.':
+        case '+':
         case '?':
         case '[':
         case ']':
+        case '{':
+        case '}':
+        case '(':
+        case ')':
+        case '\\':
           g_string_append_printf (str, "\\%c", *src);
           break;
-        case '|':
-          /* FIXME: We actually need to match :[0-9]+ or '/'. Sign means
-             "here could be port number or nothing". So bla.com^ will match
-             bla.com/ or bla.com:8080/ but not bla.com.au/ */
-        case '^':
-        case '+':
-          break;
         default:
           g_string_append_printf (str,"%c", *src);
           break;
@@ -429,11 +469,6 @@ uri_tester_fixup_regexp (const char *prefix, char *src)
     }
   while (*src);
 
-  len = str->len;
-  /* We dont need .* in the end of url. Thats stupid */
-  if (str->str && str->str[len-1] == '*' && str->str[len-2] == '.')
-    g_string_erase (str, len-2, 2);
-
   return str;
 }
 
@@ -444,15 +479,18 @@ uri_tester_compile_regexp (UriTester *tester,
                            gboolean   whitelist)
 {
   GHashTable *pattern;
+  GHashTable *keys;
   GHashTable *optslist;
   GRegex *regex;
   GError *error = NULL;
   char *patt;
+  int len;
 
   if (!gpatt)
     return;
 
   patt = gpatt->str;
+  len = gpatt->len;
 
   /* TODO: Play with optimization flags */
   regex = g_regex_new (patt, G_REGEX_OPTIMIZE | G_REGEX_JAVASCRIPT_COMPAT,
@@ -465,16 +503,55 @@ uri_tester_compile_regexp (UriTester *tester,
     }
 
   pattern = tester->priv->pattern;
+  keys = tester->priv->keys;
   optslist = tester->priv->optslist;
   if (whitelist)
     {
       pattern = tester->priv->whitelisted_pattern;
+      keys = tester->priv->whitelisted_keys;
       optslist = tester->priv->whitelisted_optslist;
     }
 
-  LOG ("patt: %s%s", patt, "");
-  g_hash_table_insert (pattern, g_strdup (patt), regex);
-  g_hash_table_insert (optslist, g_strdup (patt), g_strdup (opts));
+  if (!g_regex_match (tester->priv->regex_pattern, patt, 0, NULL))
+    {
+      int signature_count = 0;
+      int pos = 0;
+      char *sig;
+
+      for (pos = len - SIGNATURE_SIZE; pos >= 0; pos--) {
+        sig = g_strndup (patt + pos, SIGNATURE_SIZE);
+        if (!strchr (sig, '*') &&
+            !g_hash_table_lookup (keys, sig))
+          {
+            LOG ("sig: %s %s", sig, patt);
+            g_hash_table_insert (keys, g_strdup (sig), g_regex_ref (regex));
+            g_hash_table_insert (optslist, g_strdup (sig), g_strdup (opts));
+            signature_count++;
+          }
+        else
+          {
+            if (sig[0] == '*' &&
+                !g_hash_table_lookup (pattern, patt))
+              {
+                LOG ("patt2: %s %s", sig, patt);
+                g_hash_table_insert (pattern, g_strdup (patt), g_regex_ref (regex));
+                g_hash_table_insert (optslist, g_strdup (patt), g_strdup (opts));
+              }
+          }
+        g_free (sig);
+      }
+      g_regex_unref (regex);
+
+      if (signature_count > 1 && g_hash_table_lookup (pattern, patt))
+        g_hash_table_steal (pattern, patt);
+    }
+  else
+    {
+      LOG ("patt: %s%s", patt, "");
+      /* Pattern is a regexp chars */
+      g_hash_table_insert (pattern, g_strdup (patt), regex);
+      g_hash_table_insert (optslist, g_strdup (patt), g_strdup (opts));
+    }
 }
 
 static void
@@ -668,66 +745,45 @@ uri_tester_parse_line (UriTester *tester, char *line, gboolean whitelist)
 }
 
 static void
-file_parse_cb (GDataInputStream *stream, GAsyncResult *result, UriTester *tester)
+uri_tester_parse_file_at_uri (UriTester *tester, const char *fileuri)
 {
+  GFile *file;
+  GFileInputStream *stream;
+  GDataInputStream *data_stream = NULL;
   char *line;
   GError *error = NULL;
 
-  line = g_data_input_stream_read_line_finish (stream, result, NULL, &error);
-  if (!line) {
-    if (error) {
-      LOG ("Error parsing file: %s\n", error->message);
-      g_error_free (error);
-    }
+  file = g_file_new_for_uri (fileuri);
+  stream = g_file_read (file, NULL, &error);
+  g_object_unref (file);
 
-    return;
-  }
-
-  uri_tester_parse_line (tester, line, FALSE);
-  g_free (line);
-
-  g_data_input_stream_read_line_async (stream, G_PRIORITY_DEFAULT_IDLE, NULL,
-                                       (GAsyncReadyCallback)file_parse_cb, tester);
-}
-
-static void
-file_read_cb (GFile *file, GAsyncResult *result, UriTester *tester)
-{
-  GFileInputStream *stream;
-  GDataInputStream *data_stream;
-  GError *error = NULL;
-
-  stream = g_file_read_finish (file, result, &error);
   if (!stream) {
-    char *path;
-
-    path = g_file_get_path (file);
-    LOG ("Error opening file %s for parsing: %s\n", path, error->message);
-    g_free (path);
+    g_warning ("Error opening %s for parsing: %s\n", fileuri, error->message);
     g_error_free (error);
-
     return;
   }
 
   data_stream = g_data_input_stream_new (G_INPUT_STREAM (stream));
   g_object_unref (stream);
 
-  g_data_input_stream_read_line_async (data_stream, G_PRIORITY_DEFAULT_IDLE, NULL,
-                                       (GAsyncReadyCallback)file_parse_cb, tester);
+  for (;;) {
+    line = g_data_input_stream_read_line (data_stream, NULL, NULL, &error);
+    if (!line) {
+      if (error) {
+        g_warning ("Error parsing file: %s\n", error->message);
+        g_error_free (error);
+      }
+
+      break;
+    }
+
+    uri_tester_parse_line (tester, line, FALSE);
+    g_free (line);
+  }
+
   g_object_unref (data_stream);
 }
 
-static void
-uri_tester_parse_file_at_uri (UriTester *tester, const char *fileuri)
-{
-  GFile *file;
-  GFileInputStream *stream;
-
-  file = g_file_new_for_uri (fileuri);
-  g_file_read_async (file, G_PRIORITY_DEFAULT_IDLE, NULL, (GAsyncReadyCallback)file_read_cb, tester);
-  g_object_unref (file);
-}
-
 static void
 uri_tester_init (UriTester *tester)
 {
@@ -742,6 +798,9 @@ uri_tester_init (UriTester *tester)
   priv->pattern = g_hash_table_new_full (g_str_hash, g_str_equal,
                                          (GDestroyNotify)g_free,
                                          (GDestroyNotify)g_regex_unref);
+  priv->keys = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                      (GDestroyNotify)g_free,
+                                      (GDestroyNotify)g_regex_unref);
   priv->optslist = g_hash_table_new_full (g_str_hash, g_str_equal,
                                           (GDestroyNotify)g_free,
                                           (GDestroyNotify)g_free);
@@ -752,6 +811,9 @@ uri_tester_init (UriTester *tester)
   priv->whitelisted_pattern = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                        (GDestroyNotify)g_free,
                                                        (GDestroyNotify)g_regex_unref);
+  priv->whitelisted_keys = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                    (GDestroyNotify)g_free,
+                                                    (GDestroyNotify)g_regex_unref);
   priv->whitelisted_optslist = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                         (GDestroyNotify)g_free,
                                                         (GDestroyNotify)g_free);
@@ -825,10 +887,12 @@ uri_tester_finalize (GObject *object)
   g_free (priv->data_dir);
 
   g_hash_table_destroy (priv->pattern);
+  g_hash_table_destroy (priv->keys);
   g_hash_table_destroy (priv->optslist);
   g_hash_table_destroy (priv->urlcache);
 
   g_hash_table_destroy (priv->whitelisted_pattern);
+  g_hash_table_destroy (priv->whitelisted_keys);
   g_hash_table_destroy (priv->whitelisted_optslist);
   g_hash_table_destroy (priv->whitelisted_urlcache);
 
