@@ -348,6 +348,7 @@ drag_motion_cb (GtkWidget *widget,
 			g_timeout_add (150, 
 				       scroll_timeout, 
 				       GTK_TREE_VIEW (view));
+		g_source_set_name_by_id (view->priv->scroll_id, "[epiphany] scroll_timeout");
 	}
 
 	gdk_drag_status (context, action, time);
@@ -1268,12 +1269,14 @@ ephy_node_view_add_data_column (EphyNodeView *view,
 }
 
 /**
- * ephy_node_view_add_column:
+ * ephy_node_view_add_column_full:
  * @view: an #EphyNodeView
  * @title: title for the column
  * @value_type: type to be held by the column
  * @prop_id: numeric id corresponding to the column in the model to be shown
  * @flags: flags for the new column
+ * @func: optional function to modify the view of properties in the column
+ * @user_data: optional data passed to @func
  * @icon_func: a function providing the icon for the column
  * @ret: location to store the created column
  *
@@ -1282,29 +1285,30 @@ ephy_node_view_add_data_column (EphyNodeView *view,
  * Returns: the id of the new column
  **/
 int
-ephy_node_view_add_column (EphyNodeView *view,
-			   const char  *title,
-			   GType value_type,
-			   guint prop_id,
-			   EphyNodeViewFlags flags,
-			   EphyTreeModelNodeValueFunc icon_func,
-			   GtkTreeViewColumn **ret)
-
+ephy_node_view_add_column_full (EphyNodeView *view,
+				const char  *title,
+				GType value_type,
+				guint prop_id,
+				EphyNodeViewFlags flags,
+				EphyTreeModelNodeValueFunc func,
+				gpointer user_data,
+				EphyTreeModelNodeValueFunc icon_func,
+				GtkTreeViewColumn **ret)
 {
 	GtkTreeViewColumn *gcolumn;
 	GtkCellRenderer *renderer;
 	int column;
 	int icon_column;
 
-	column = ephy_tree_model_node_add_prop_column
-		(view->priv->nodemodel, value_type, prop_id);
+	column = ephy_tree_model_node_add_column_full
+		(view->priv->nodemodel, value_type, prop_id, func, user_data);
 
 	gcolumn = (GtkTreeViewColumn *) gtk_tree_view_column_new ();
 
 	if (icon_func)
 	{
 		icon_column = ephy_tree_model_node_add_func_column
-			 (view->priv->nodemodel, GDK_TYPE_PIXBUF, icon_func, NULL);
+			 (view->priv->nodemodel, GDK_TYPE_PIXBUF, icon_func, view);
 
 		renderer = gtk_cell_renderer_pixbuf_new ();
 		gtk_tree_view_column_pack_start (gcolumn, renderer, FALSE);
@@ -1380,6 +1384,33 @@ ephy_node_view_add_column (EphyNodeView *view,
 	return column;
 }
 
+/**
+ * ephy_node_view_add_column:
+ * @view: an #EphyNodeView
+ * @title: title for the column
+ * @value_type: type to be held by the column
+ * @prop_id: numeric id corresponding to the column in the model to be shown
+ * @flags: flags for the new column
+ * @icon_func: a function providing the icon for the column
+ * @ret: location to store the created column
+ *
+ * Adds a new column, corresponding to a @prop_id of the model, to the @view.
+ *
+ * Returns: the id of the new column
+ **/
+int
+ephy_node_view_add_column (EphyNodeView *view,
+			   const char  *title,
+			   GType value_type,
+			   guint prop_id,
+			   EphyNodeViewFlags flags,
+			   EphyTreeModelNodeValueFunc icon_func,
+			   GtkTreeViewColumn **ret)
+{
+	return ephy_node_view_add_column_full (view, title, value_type, prop_id,
+					       flags, NULL, NULL, icon_func, ret);
+}
+
 /**
  * ephy_node_view_set_priority:
  * @view: an #EphyNodeView
@@ -1809,6 +1840,25 @@ ephy_node_view_popup (EphyNodeView *view, GtkWidget *menu)
 	}
 }
 
+gboolean
+ephy_node_view_get_iter_for_node (EphyNodeView *view,
+				  GtkTreeIter *iter,
+				  EphyNode *node)
+{
+	GtkTreeIter node_iter, filter_iter;
+
+	ephy_tree_model_node_iter_from_node (EPHY_TREE_MODEL_NODE (view->priv->nodemodel), node, &node_iter);
+	if (!gtk_tree_model_filter_convert_child_iter_to_iter (GTK_TREE_MODEL_FILTER (view->priv->filtermodel),
+							       &filter_iter, &node_iter))
+		return FALSE;
+
+	if (!gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (view->priv->sortmodel),
+							     iter, &filter_iter))
+		return FALSE;
+
+	return TRUE;
+}
+
 static void
 ephy_node_view_class_init (EphyNodeViewClass *klass)
 {
